"use strict";
// Copyright 2021 Google Inc. Use of this source code is governed by an
// MIT-style license that can be found in the LICENSE file or at
// https://opensource.org/licenses/MIT.
Object.defineProperty(exports, "__esModule", { value: true });
exports.SassColor = void 0;
const value_1 = require("./value");
const utils_1 = require("./utils");
const immutable_1 = require("immutable");
/** A SassScript color. */
class SassColor extends value_1.Value {
    constructor(color) {
        super();
        if ('red' in color) {
            this.redInternal = color.red;
            this.greenInternal = color.green;
            this.blueInternal = color.blue;
        }
        else {
            this.hueInternal = color.hue;
            this.saturationInternal = color.saturation;
            this.lightnessInternal = color.lightness;
        }
        this.alphaInternal = color.alpha;
    }
    /**
     * Creates an RGB color.
     *
     * Throws an error if `red`, `green`, and `blue` aren't between `0` and `255`,
     * or if `alpha` isn't between `0` and `1`.
     */
    static rgb(red, green, blue, alpha) {
        return new SassColor({
            red: (0, utils_1.fuzzyAssertInRange)(red, 0, 255, 'red'),
            green: (0, utils_1.fuzzyAssertInRange)(green, 0, 255, 'green'),
            blue: (0, utils_1.fuzzyAssertInRange)(blue, 0, 255, 'blue'),
            alpha: alpha === undefined ? 1 : (0, utils_1.fuzzyAssertInRange)(alpha, 0, 1, 'alpha'),
        });
    }
    /**
     * Creates an HSL color.
     *
     * Throws an error if `saturation` or `lightness` aren't between `0` and
     * `100`, or if `alpha` isn't between `0` and `1`.
     */
    static hsl(hue, saturation, lightness, alpha) {
        return new SassColor({
            hue: hue % 360,
            saturation: (0, utils_1.fuzzyAssertInRange)(saturation, 0, 100, 'saturation'),
            lightness: (0, utils_1.fuzzyAssertInRange)(lightness, 0, 100, 'lightness'),
            alpha: alpha === undefined ? 1 : (0, utils_1.fuzzyAssertInRange)(alpha, 0, 1, 'alpha'),
        });
    }
    /**
     * Creates an HWB color.
     *
     * Throws an error if `whiteness` or `blackness` aren't between `0` and `100`,
     * or if `alpha` isn't between `0` and `1`.
     */
    static hwb(hue, whiteness, blackness, alpha) {
        // From https://www.w3.org/TR/css-color-4/#hwb-to-rgb
        const scaledHue = (hue % 360) / 360;
        let scaledWhiteness = (0, utils_1.fuzzyAssertInRange)(whiteness, 0, 100, 'whiteness') / 100;
        let scaledBlackness = (0, utils_1.fuzzyAssertInRange)(blackness, 0, 100, 'blackness') / 100;
        const sum = scaledWhiteness + scaledBlackness;
        if (sum > 1) {
            scaledWhiteness /= sum;
            scaledBlackness /= sum;
        }
        const factor = 1 - scaledWhiteness - scaledBlackness;
        function toRgb(hue) {
            const channel = hueToRgb(0, 1, hue) * factor + scaledWhiteness;
            return (0, utils_1.fuzzyRound)(channel * 255);
        }
        // Because HWB is (currently) used much less frequently than HSL or RGB, we
        // don't cache its values because we expect the memory overhead of doing so
        // to outweigh the cost of recalculating it on access. Instead, we eagerly
        // convert it to RGB and then convert back if necessary.
        return SassColor.rgb(toRgb(scaledHue + 1 / 3), toRgb(scaledHue), toRgb(scaledHue - 1 / 3), alpha);
    }
    /** `this`'s red channel. */
    get red() {
        if (this.redInternal === undefined) {
            this.hslToRgb();
        }
        return this.redInternal;
    }
    /** `this`'s blue channel. */
    get blue() {
        if (this.blueInternal === undefined) {
            this.hslToRgb();
        }
        return this.blueInternal;
    }
    /** `this`'s green channel. */
    get green() {
        if (this.greenInternal === undefined) {
            this.hslToRgb();
        }
        return this.greenInternal;
    }
    /** `this`'s hue value. */
    get hue() {
        if (this.hueInternal === undefined) {
            this.rgbToHsl();
        }
        return this.hueInternal;
    }
    /** `this`'s saturation value. */
    get saturation() {
        if (this.saturationInternal === undefined) {
            this.rgbToHsl();
        }
        return this.saturationInternal;
    }
    /** `this`'s hue value. */
    get lightness() {
        if (this.lightnessInternal === undefined) {
            this.rgbToHsl();
        }
        return this.lightnessInternal;
    }
    /** `this`'s whiteness value. */
    get whiteness() {
        // Because HWB is (currently) used much less frequently than HSL or RGB, we
        // don't cache its values because we expect the memory overhead of doing so
        // to outweigh the cost of recalculating it on access.
        return (Math.min(this.red, this.green, this.blue) / 255) * 100;
    }
    /** `this`'s blackness value. */
    get blackness() {
        // Because HWB is (currently) used much less frequently than HSL or RGB, we
        // don't cache its values because we expect the memory overhead of doing so
        // to outweigh the cost of recalculating it on access.
        return 100 - (Math.max(this.red, this.green, this.blue) / 255) * 100;
    }
    /** `this`'s alpha channel. */
    get alpha() {
        return this.alphaInternal;
    }
    assertColor() {
        return this;
    }
    /**
     * Returns a copy of `this` with its RGB channels changed to `red`, `green`,
     * `blue`, and/or `alpha`.
     */
    changeRgb(options) {
        var _a, _b, _c, _d;
        return SassColor.rgb((_a = options.red) !== null && _a !== void 0 ? _a : this.red, (_b = options.green) !== null && _b !== void 0 ? _b : this.green, (_c = options.blue) !== null && _c !== void 0 ? _c : this.blue, (_d = options.alpha) !== null && _d !== void 0 ? _d : this.alpha);
    }
    /**
     * Returns a copy of `this` with its HSL values changed to `hue`,
     * `saturation`, `lightness`, and/or `alpha`.
     */
    changeHsl(options) {
        var _a, _b, _c, _d;
        return SassColor.hsl((_a = options.hue) !== null && _a !== void 0 ? _a : this.hue, (_b = options.saturation) !== null && _b !== void 0 ? _b : this.saturation, (_c = options.lightness) !== null && _c !== void 0 ? _c : this.lightness, (_d = options.alpha) !== null && _d !== void 0 ? _d : this.alpha);
    }
    /**
     * Returns a copy of `this` with its HWB values changed to `hue`, `whiteness`,
     * `blackness`, and/or `alpha`.
     */
    changeHwb(options) {
        var _a, _b, _c, _d;
        return SassColor.hwb((_a = options.hue) !== null && _a !== void 0 ? _a : this.hue, (_b = options.whiteness) !== null && _b !== void 0 ? _b : this.whiteness, (_c = options.blackness) !== null && _c !== void 0 ? _c : this.blackness, (_d = options.alpha) !== null && _d !== void 0 ? _d : this.alpha);
    }
    /** Returns a copy of `this` with its alpha channel changed to `alpha`. */
    changeAlpha(alpha) {
        return new SassColor({
            red: this.red,
            green: this.green,
            blue: this.blue,
            alpha: (0, utils_1.fuzzyAssertInRange)(alpha, 0, 1, 'alpha'),
        });
    }
    equals(other) {
        return (other instanceof SassColor &&
            (0, utils_1.fuzzyEquals)(this.red, other.red) &&
            (0, utils_1.fuzzyEquals)(this.green, other.green) &&
            (0, utils_1.fuzzyEquals)(this.blue, other.blue) &&
            (0, utils_1.fuzzyEquals)(this.alpha, other.alpha));
    }
    hashCode() {
        return (0, immutable_1.hash)(this.red ^ this.green ^ this.blue ^ this.alpha);
    }
    toString() {
        const isOpaque = (0, utils_1.fuzzyEquals)(this.alpha, 1);
        let string = isOpaque ? 'rgb(' : 'rgba(';
        string += `${this.red}, ${this.green}, ${this.blue}`;
        string += isOpaque ? ')' : `, ${this.alpha})`;
        return string;
    }
    // Computes `this`'s `hue`, `saturation`, and `lightness` values based on
    // `red`, `green`, and `blue`.
    //
    // Algorithm from https://en.wikipedia.org/wiki/HSL_and_HSV#RGB_to_HSL_and_HSV
    rgbToHsl() {
        const scaledRed = this.red / 255;
        const scaledGreen = this.green / 255;
        const scaledBlue = this.blue / 255;
        const max = Math.max(scaledRed, scaledGreen, scaledBlue);
        const min = Math.min(scaledRed, scaledGreen, scaledBlue);
        const delta = max - min;
        if (max === min) {
            this.hueInternal = 0;
        }
        else if (max === scaledRed) {
            this.hueInternal = ((60 * (scaledGreen - scaledBlue)) / delta) % 360;
        }
        else if (max === scaledGreen) {
            this.hueInternal = (120 + (60 * (scaledBlue - scaledRed)) / delta) % 360;
        }
        else if (max === scaledBlue) {
            this.hueInternal = (240 + (60 * (scaledRed - scaledGreen)) / delta) % 360;
        }
        this.lightnessInternal = 50 * (max + min);
        if (max === min) {
            this.saturationInternal = 0;
        }
        else if (this.lightnessInternal < 50) {
            this.saturationInternal = (100 * delta) / (max + min);
        }
        else {
            this.saturationInternal = (100 * delta) / (2 - max - min);
        }
    }
    // Computes `this`'s red`, `green`, and `blue` channels based on `hue`,
    // `saturation`, and `value`.
    //
    // Algorithm from the CSS3 spec: https://www.w3.org/TR/css3-color/#hsl-color.
    hslToRgb() {
        const scaledHue = this.hue / 360;
        const scaledSaturation = this.saturation / 100;
        const scaledLightness = this.lightness / 100;
        const m2 = scaledLightness <= 0.5
            ? scaledLightness * (scaledSaturation + 1)
            : scaledLightness +
                scaledSaturation -
                scaledLightness * scaledSaturation;
        const m1 = scaledLightness * 2 - m2;
        this.redInternal = (0, utils_1.fuzzyRound)(hueToRgb(m1, m2, scaledHue + 1 / 3) * 255);
        this.greenInternal = (0, utils_1.fuzzyRound)(hueToRgb(m1, m2, scaledHue) * 255);
        this.blueInternal = (0, utils_1.fuzzyRound)(hueToRgb(m1, m2, scaledHue - 1 / 3) * 255);
    }
}
exports.SassColor = SassColor;
// An algorithm from the CSS3 spec: http://www.w3.org/TR/css3-color/#hsl-color.
function hueToRgb(m1, m2, hue) {
    if (hue < 0)
        hue += 1;
    if (hue > 1)
        hue -= 1;
    if (hue < 1 / 6) {
        return m1 + (m2 - m1) * hue * 6;
    }
    else if (hue < 1 / 2) {
        return m2;
    }
    else if (hue < 2 / 3) {
        return m1 + (m2 - m1) * (2 / 3 - hue) * 6;
    }
    else {
        return m1;
    }
}
//# sourceMappingURL=color.js.map